# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='NoticeQueueBatch',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('pickled_data', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='NoticeSetting',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('medium', models.CharField(verbose_name='medium', choices=[(0, 'email')], max_length=1)),
                ('send', models.BooleanField(verbose_name='send', default=False)),
            ],
            options={
                'verbose_name': 'notice setting',
                'verbose_name_plural': 'notice settings',
            },
        ),
        migrations.CreateModel(
            name='NoticeType',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(verbose_name='label', max_length=40)),
                ('display', models.CharField(verbose_name='display', max_length=50)),
                ('description', models.CharField(verbose_name='description', max_length=100)),
                ('default', models.IntegerField(verbose_name='default')),
            ],
            options={
                'verbose_name': 'notice type',
                'verbose_name_plural': 'notice types',
            },
        ),
        migrations.AddField(
            model_name='noticesetting',
            name='notice_type',
            field=models.ForeignKey(verbose_name='notice type', to='pinax_notifications.NoticeType'),
        ),
        migrations.AddField(
            model_name='noticesetting',
            name='user',
            field=models.ForeignKey(verbose_name='user', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='noticesetting',
            unique_together=set([('user', 'notice_type', 'medium')]),
        ),
    ]



import base64

from django.core.exceptions import ObjectDoesNotExist
from django.db.models.query import QuerySet
from django.utils.six.moves import cPickle as pickle

from .conf import settings
from .utils import load_media_defaults


class DefaultHookSet(object):

    def notice_setting_for_user(self, user, notice_type, medium):
        kwargs = {
            "notice_type": notice_type,
            "medium": medium
        }
        try:
            return user.noticesetting_set.get(**kwargs)
        except ObjectDoesNotExist:
            _, NOTICE_MEDIA_DEFAULTS = load_media_defaults()
            default = (NOTICE_MEDIA_DEFAULTS[medium] <= notice_type.default)
            kwargs.update({"send": default})
            setting = user.noticesetting_set.create(**kwargs)
            return setting

    def queue(self, users, label, extra_context=None, sender=None):
        """
        Queue the notification in NoticeQueueBatch. This allows for large amounts
        of user notifications to be deferred to a seperate process running outside
        the webserver.
        """
        from pinax.notifications.models import NoticeQueueBatch
        if extra_context is None:
            extra_context = {}
        if isinstance(users, QuerySet):
            users = [row["pk"] for row in users.values("pk")]
        else:
            users = [user.pk for user in users]
        notices = []
        for user in users:
            notices.append((user, label, extra_context, sender))
        NoticeQueueBatch(pickled_data=base64.b64encode(pickle.dumps(notices))).save()


class HookProxy(object):

    def __getattr__(self, attr):
        return getattr(settings.PINAX_NOTIFICATIONS_HOOKSET, attr)


hookset = HookProxy()



from .conf import settings


def load_media_defaults():
    media = []
    defaults = {}
    for key, backend in settings.PINAX_NOTIFICATIONS_BACKENDS.items():
        # key is a tuple (medium_id, backend_label)
        media.append(key)
        defaults[key[0]] = backend.spam_sensitivity
    return media, defaults
